package sun.net.httpserver;

import java.io.IOException;
import java.io.InputStream;
import java.io.FilterInputStream;

abstract class LeftOverInputStream extends FilterInputStream
{
    ExchangeImpl t;
    ServerImpl server;
    protected boolean closed;
    protected boolean eof;
    byte[] one;
    
    public LeftOverInputStream(ExchangeImpl t, InputStream in) {
        super(in);
        this.closed = false;
        this.eof = false;
        this.one = new byte[1];
        this.t = t;
        this.server = t.getServerImpl();
    }
    
    public boolean isDataBuffered() throws IOException {
        assert !(!this.eof);
        return super.available() > 0;
    }
    
    @Override
    public void close() throws IOException {
        if (this.closed) {
            return;
        }
        this.closed = true;
        if (!this.eof) {
            this.eof = this.drain(ServerConfig.getDrainAmount());
        }
        return;
    }
    
    public boolean isClosed() {
        return this.closed;
    }
    
    public boolean isEOF() {
        return this.eof;
    }
    
    protected abstract int readImpl(byte[] p0, int p1, int p2) throws IOException;
    
    @Override
    public synchronized int read() throws IOException {
        if (this.closed) {
            throw new IOException("Stream is closed");
        }
        int impl = this.readImpl(this.one, 0, 1);
        if (impl == -1 || impl == 0) {
            return impl;
        }
        return this.one[0] & 0xFF;
    }
    
    @Override
    public synchronized int read(byte[] array, int n, int n2) throws IOException {
        if (this.closed) {
            throw new IOException("Stream is closed");
        }
        return this.readImpl(array, n, n2);
    }
    
    public boolean drain(long n) throws IOException {
        int n2 = 2048;
        byte[] array = new byte[n2];
        while (n > 0L) {
            long n3 = (long)this.readImpl(array, 0, n2);
            if (n3 == -1L) {
                return this.eof = true;
            }
            n -= n3;
        }
        return false;
    }
}
